@@ -32,7 +32,8 @@ class DataSet(object):
 
         # Introspect the database and generate models.
         self._introspector = Introspector.from_database(self._database)
-        self._models = self._introspector.generate_models(skip_invalid=True)
+        self._models = self._introspector.generate_models(
+            skip_invalid=True, literal_column_names=True)
         self._migrator = SchemaMigrator.from_database(self._database)
 
         class BaseModel(Model):
@@ -56,6 +57,8 @@ class DataSet(object):
             'json': JSONImporter}
 
     def __getitem__(self, table):
+        if table not in self._models and table in self.tables:
+            self.update_cache(table)
         return Table(self, table, self._models.get(table))
 
     @property
@@ -73,13 +76,18 @@ class DataSet(object):
 
     def update_cache(self, table=None):
         if table:
-            model_class = self._models[table]
-            dependencies = model_class._meta.related_models(backrefs=True)
+            dependencies = [table]
+            if table in self._models:
+                model_class = self._models[table]
+                dependencies.extend([
+                    related._meta.db_table for related in
+                    model_class._meta.related_models(backrefs=True)])
         else:
             dependencies = None  # Update all tables.
         updated = self._introspector.generate_models(
             skip_invalid=True,
-            table_names=[related._meta.db_table for related in dependencies])
+            table_names=dependencies,
+            literal_column_names=True)
         self._models.update(updated)
 
     def __enter__(self):
@@ -161,7 +169,12 @@ class Table(object):
         return iter(self.find().iterator())
 
     def _create_model(self):
-        return type(str(self.name), (self.dataset._base_model,), {})
+        class Meta:
+            db_table = self.name
+        return type(
+            str(self.name),
+            (self.dataset._base_model,),
+            {'Meta': Meta})
 
     def create_index(self, columns, unique=False):
         self.dataset._database.create_index(
@@ -186,7 +199,7 @@ class Table(object):
 
     @property
     def columns(self):
-        return self.model_class._meta.get_field_names()
+        return [f.name for f in self.model_class._meta.sorted_fields]
 
     def _migrate_new_columns(self, data):
         new_keys = set(data) - set(self.model_class._meta.fields)
@@ -304,7 +317,6 @@ class JSONImporter(Importer):
     def load(self, file_obj, **kwargs):
         data = json.load(file_obj, **kwargs)
         count = 0
-        valid_columns = set(self.columns)
 
         for row in data:
             if self.strict:
@@ -341,7 +353,7 @@ class CSVImporter(Importer):
             else:
                 header_fields = list(enumerate(header_keys))
         else:
-            header_fields = list(enumerate(self.model._meta.get_fields()))
+            header_fields = list(enumerate(self.model._meta.sorted_fields))
 
         if not header_fields:
             return count
