@@ -6,11 +6,12 @@ Collection of postgres-specific extensions, currently including:
 import uuid
 
 from peewee import *
-from peewee import coerce_to_unicode
 from peewee import Expression
 from peewee import logger
 from peewee import Node
+from peewee import OP
 from peewee import Param
+from peewee import Passthrough
 from peewee import returns_clone
 from peewee import QueryCompiler
 from peewee import SelectQuery
@@ -22,7 +23,6 @@ try:
 except ImportError:
     pass
 
-from psycopg2 import extensions
 from psycopg2.extensions import adapt
 from psycopg2.extensions import AsIs
 from psycopg2.extensions import register_adapter
@@ -32,6 +32,9 @@ try:
 except:
     Json = None
 
+@Node.extend(clone=False)
+def cast(self, as_type):
+    return Expression(self, OP.CAST, SQL(as_type))
 
 class _LookupNode(Node):
     def __init__(self, node, parts):
@@ -42,6 +45,9 @@ class _LookupNode(Node):
     def clone_base(self):
         return type(self)(self.node, list(self.parts))
 
+    def cast(self, as_type):
+        return Expression(Clause(self, parens=True), OP.CAST, SQL(as_type))
+
 class _JsonLookupBase(_LookupNode):
     def __init__(self, node, parts, as_json=False):
         super(_JsonLookupBase, self).__init__(node, parts)
@@ -54,6 +60,24 @@ class _JsonLookupBase(_LookupNode):
     def as_json(self, as_json=True):
         self._as_json = as_json
 
+    def contains(self, other):
+        clone = self.as_json(True)
+        if isinstance(other, (list, dict)):
+            return Expression(clone, OP.JSONB_CONTAINS, Json(other))
+        return Expression(clone, OP.JSONB_EXISTS, other)
+
+    def contains_any(self, *keys):
+        return Expression(
+            self.as_json(True),
+            OP.JSONB_CONTAINS_ANY_KEY,
+            Passthrough(list(keys)))
+
+    def contains_all(self, *keys):
+        return Expression(
+            self.as_json(True),
+            OP.JSONB_CONTAINS_ALL_KEYS,
+            Passthrough(list(keys)))
+
 class JsonLookup(_JsonLookupBase):
     _node_type = 'json_lookup'
 
@@ -79,11 +103,10 @@ class ObjectSlice(_LookupNode):
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
 
-class _Array(Node):
+class _Array(object):
     def __init__(self, field, items):
         self.field = field
         self.items = items
-        super(_Array, self).__init__()
 
 def adapt_array(arr):
     conn = arr.field.model_class._meta.database.get_conn()
@@ -92,20 +115,23 @@ def adapt_array(arr):
     return AsIs('%s::%s%s' % (
         items,
         arr.field.get_column_type(),
-        '[]'* arr.field.dimensions))
+        '[]' * arr.field.dimensions))
 register_adapter(_Array, adapt_array)
 
 
-class IndexedField(Field):
+class IndexedFieldMixin(object):
     default_index_type = 'GiST'
 
     def __init__(self, index_type=None, *args, **kwargs):
         kwargs.setdefault('index', True)  # By default, use an index.
-        super(IndexedField, self).__init__(*args, **kwargs)
-        self.index_type = index_type or self.default_index_type
+        super(IndexedFieldMixin, self).__init__(*args, **kwargs)
+        if self.index:
+            self.index_type = index_type or self.default_index_type
+        else:
+            self.index_type = None
 
 
-class ArrayField(IndexedField):
+class ArrayField(IndexedFieldMixin, Field):
     default_index_type = 'GIN'
 
     def __init__(self, field_class=IntegerField, dimensions=1, *args,
@@ -120,25 +146,36 @@ class ArrayField(IndexedField):
         sql.value += '[]' * self.dimensions
         return sql
 
+    def db_value(self, value):
+        if value is None:
+            return
+        if not isinstance(value, (list, _Array)):
+            value = list(value)
+        return _Array(self, value)
+
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
 
     def contains(self, *items):
-        return Expression(self, OP_ACONTAINS, _Array(self, list(items)))
+        return Expression(self, OP.ACONTAINS, Param(items))
 
     def contains_any(self, *items):
-        return Expression(self, OP_ACONTAINS_ANY, _Array(self, list(items)))
+        return Expression(self, OP.ACONTAINS_ANY, Param(items))
 
 
 class DateTimeTZField(DateTimeField):
-    db_field = 'datetime_tz'
+    db_field = 'timestamptz'
 
 
-class HStoreField(IndexedField):
+class IntervalField(Field):
+    db_field = 'interval'
+
+
+class HStoreField(IndexedFieldMixin, Field):
     db_field = 'hash'
 
     def __getitem__(self, key):
-        return Expression(self, OP_HKEY, Param(key))
+        return Expression(self, OP.HKEY, Param(key))
 
     def keys(self):
         return fn.akeys(self)
@@ -150,7 +187,7 @@ class HStoreField(IndexedField):
         return fn.hstore_to_matrix(self)
 
     def slice(self, *args):
-        return fn.slice(self, Param(list(args)))
+        return fn.slice(self, Passthrough(list(args)))
 
     def exists(self, key):
         return fn.exist(self, key)
@@ -159,20 +196,20 @@ class HStoreField(IndexedField):
         return fn.defined(self, key)
 
     def update(self, **data):
-        return Expression(self, OP_HUPDATE, data)
+        return Expression(self, OP.HUPDATE, data)
 
     def delete(self, *keys):
-        return fn.delete(self, Param(list(keys)))
+        return fn.delete(self, Passthrough(list(keys)))
 
     def contains(self, value):
         if isinstance(value, dict):
-            return Expression(self, OP_HCONTAINS_DICT, Param(value))
+            return Expression(self, OP.HCONTAINS_DICT, Passthrough(value))
         elif isinstance(value, (list, tuple)):
-            return Expression(self, OP_HCONTAINS_KEYS, Param(value))
-        return Expression(self, OP_HCONTAINS_KEY, value)
+            return Expression(self, OP.HCONTAINS_KEYS, Passthrough(value))
+        return Expression(self, OP.HCONTAINS_KEY, value)
 
     def contains_any(self, *keys):
-        return Expression(self, OP_HCONTAINS_ANY_KEY, Param(value))
+        return Expression(self, OP.HCONTAINS_ANY_KEY, Passthrough(list(keys)))
 
 
 class JSONField(Field):
@@ -186,7 +223,11 @@ class JSONField(Field):
         super(JSONField, self).__init__(*args, **kwargs)
 
     def db_value(self, value):
-        return Json(value, dumps=self.dumps)
+        if value is None:
+            return value
+        if not isinstance(value, Json):
+            return Json(value, dumps=self.dumps)
+        return value
 
     def __getitem__(self, value):
         return JsonLookup(self, [value])
@@ -195,30 +236,65 @@ class JSONField(Field):
         return JsonPath(self, keys)
 
 
-class TSVectorField(IndexedField):
+class BinaryJSONField(IndexedFieldMixin, JSONField):
+    db_field = 'jsonb'
+    default_index_type = 'GIN'
+
+    def contains(self, other):
+        if isinstance(other, (list, dict)):
+            return Expression(self, OP.JSONB_CONTAINS, Json(other))
+        return Expression(self, OP.JSONB_EXISTS, Passthrough(other))
+
+    def contained_by(self, other):
+        return Expression(self, OP.JSONB_CONTAINED_BY, Json(other))
+
+    def contains_any(self, *items):
+        return Expression(
+            self,
+            OP.JSONB_CONTAINS_ANY_KEY,
+            Passthrough(list(items)),)
+
+    def contains_all(self, *items):
+        return Expression(
+            self,
+            OP.JSONB_CONTAINS_ALL_KEYS,
+            Passthrough(list(items)))
+
+
+class TSVectorField(IndexedFieldMixin, TextField):
     db_field = 'tsvector'
     default_index_type = 'GIN'
 
-    def coerce(self, value):
-        return coerce_to_unicode(value or '')
-
-    def match(self, query):
-        return Expression(self, OP_TS_MATCH, fn.to_tsquery(query))
+    def match(self, query, language=None):
+        params = (language, query) if language is not None else (query,)
+        return Expression(self, OP.TS_MATCH, fn.to_tsquery(*params))
 
 
-def Match(field, query):
-    return Expression(fn.to_tsvector(field), OP_TS_MATCH, fn.to_tsquery(query))
+def Match(field, query, language=None):
+    params = (language, query) if language is not None else (query,)
+    return Expression(
+        fn.to_tsvector(field),
+        OP.TS_MATCH,
+        fn.to_tsquery(*params))
 
 
-OP_HKEY = 'key'
-OP_HUPDATE = 'H@>'
-OP_HCONTAINS_DICT = 'H?&'
-OP_HCONTAINS_KEYS = 'H?'
-OP_HCONTAINS_KEY = 'H?|'
-OP_HCONTAINS_ANY_KEY = 'H||'
-OP_ACONTAINS = 'A@>'
-OP_ACONTAINS_ANY = 'A||'
-OP_TS_MATCH = 'T@@'
+OP.update(
+    HKEY='key',
+    HUPDATE='H@>',
+    HCONTAINS_DICT='H?&',
+    HCONTAINS_KEYS='H?',
+    HCONTAINS_KEY='H?|',
+    HCONTAINS_ANY_KEY='H||',
+    ACONTAINS='A@>',
+    ACONTAINS_ANY='A||',
+    TS_MATCH='T@@',
+    JSONB_CONTAINS='JB@>',
+    JSONB_CONTAINED_BY='JB<@',
+    JSONB_CONTAINS_ANY_KEY='JB?|',
+    JSONB_CONTAINS_ALL_KEYS='JB?&',
+    JSONB_EXISTS='JB?',
+    CAST='::',
+)
 
 
 class PostgresqlExtCompiler(QueryCompiler):
@@ -229,7 +305,7 @@ class PostgresqlExtCompiler(QueryCompiler):
         # may want to use GiST indexes, for example.
         index_type = None
         for field in fields:
-            if isinstance(field, IndexedField):
+            if isinstance(field, IndexedFieldMixin):
                 index_type = field.index_type
         if index_type:
             clause.nodes.insert(-1, SQL('USING %s' % index_type))
@@ -287,7 +363,9 @@ class PostgresqlExtDatabase(PostgresqlDatabase):
         super(PostgresqlExtDatabase, self).__init__(*args, **kwargs)
 
     def get_cursor(self, name=None):
-        return self.get_conn().cursor(name=name)
+        if name:
+            return self.get_conn().cursor(name=name)
+        return self.get_conn().cursor()
 
     def execute_sql(self, sql, params=None, require_commit=True,
                     named_cursor=False):
@@ -295,18 +373,18 @@ class PostgresqlExtDatabase(PostgresqlDatabase):
         use_named_cursor = (named_cursor or (
                             self.server_side_cursors and
                             sql.lower().startswith('select')))
-        with self.exception_wrapper():
+        with self.exception_wrapper:
             if use_named_cursor:
                 cursor = self.get_cursor(name=str(uuid.uuid1()))
                 require_commit = False
             else:
                 cursor = self.get_cursor()
             try:
-                res = cursor.execute(sql, params or ())
+                cursor.execute(sql, params or ())
             except Exception as exc:
-                logger.exception('%s %s', sql, params)
-                if self.sql_error_handler(exc, sql, params, require_commit):
-                    raise
+                if self.get_autocommit() and self.autorollback:
+                    self.rollback()
+                raise
             else:
                 if require_commit and self.get_autocommit():
                     self.commit()
@@ -335,18 +413,25 @@ PostgresqlExtDatabase.register_fields({
     'datetime_tz': 'timestamp with time zone',
     'hash': 'hstore',
     'json': 'json',
+    'jsonb': 'jsonb',
     'tsvector': 'tsvector',
 })
 PostgresqlExtDatabase.register_ops({
-    OP_HCONTAINS_DICT: '@>',
-    OP_HCONTAINS_KEYS: '?&',
-    OP_HCONTAINS_KEY: '?',
-    OP_HCONTAINS_ANY_KEY: '?|',
-    OP_HKEY: '->',
-    OP_HUPDATE: '||',
-    OP_ACONTAINS: '@>',
-    OP_ACONTAINS_ANY: '&&',
-    OP_TS_MATCH: '@@',
+    OP.HCONTAINS_DICT: '@>',
+    OP.HCONTAINS_KEYS: '?&',
+    OP.HCONTAINS_KEY: '?',
+    OP.HCONTAINS_ANY_KEY: '?|',
+    OP.HKEY: '->',
+    OP.HUPDATE: '||',
+    OP.ACONTAINS: '@>',
+    OP.ACONTAINS_ANY: '&&',
+    OP.TS_MATCH: '@@',
+    OP.JSONB_CONTAINS: '@>',
+    OP.JSONB_CONTAINED_BY: '<@',
+    OP.JSONB_CONTAINS_ANY_KEY: '?|',
+    OP.JSONB_CONTAINS_ALL_KEYS: '?&',
+    OP.JSONB_EXISTS: '?',
+    OP.CAST: '::',
 })
 
 def ServerSide(select_query):
@@ -362,3 +447,11 @@ def ServerSide(select_query):
         # Expose generator for iterating over query.
         for obj in query_result.iterator():
             yield obj
+
+
+def LateralJoin(lhs, rhs, join_type='LEFT', condition=True):
+    return Clause(
+        lhs,
+        SQL('%s JOIN LATERAL' % join_type),
+        rhs,
+        SQL('ON %s', condition))
