@@ -50,9 +50,12 @@ def get_option_parser():
     cases.add_option('--reflection', dest='reflection', default=False, action='store_true', help='reflection schema introspector')
     cases.add_option('--signals', dest='signals', default=False, action='store_true', help='signals tests')
     cases.add_option('--shortcuts', dest='shortcuts', default=False, action='store_true', help='shortcuts tests')
+    cases.add_option('--speedups', dest='speedups', default=False, action='store_true', help='speedups c extension tests')
     cases.add_option('--sqlcipher-ext', dest='sqlcipher', default=False, action='store_true', help='sqlcipher_ext tests (requires pysqlcipher)')
+    cases.add_option('--sqliteq', dest='sqliteq', default=False, action='store_true', help='sqliteq tests')
     cases.add_option('--sqlite-c-ext', dest='sqlite_c', default=False, action='store_true', help='sqlite c extension tests')
     cases.add_option('--sqlite-ext', dest='sqlite_ext', default=False, action='store_true', help='sqlite_ext tests')
+    cases.add_option('--sqlite-udf', dest='sqlite_udf', default=False, action='store_true', help='sqlite_udf tests')
     cases.add_option('--test-utils', dest='test_utils', default=False, action='store_true', help='test_utils tests')
 
     parser.add_option_group(basic)
@@ -147,12 +150,21 @@ def collect_modules(options):
     if xtra(options.shortcuts):
         from playhouse.tests import test_shortcuts
         modules.append(test_shortcuts)
+    if xtra(options.speedups):
+        try:
+            from playhouse.tests import test_speedups
+            modules.append(test_speedups)
+        except ImportError:
+            print_('Unable to import speedups tests, skipping')
     if xtra(options.sqlcipher):
         try:
             from playhouse.tests import test_sqlcipher_ext
             modules.append(test_sqlcipher_ext)
         except ImportError:
             print_('Unable to import pysqlcipher tests, skipping')
+    if xtra(options.sqliteq):
+        from playhouse.tests import test_sqliteq
+        modules.append(test_sqliteq)
     if xtra(options.sqlite_c):
         try:
             from playhouse.tests import test_sqlite_c_ext
@@ -162,6 +174,9 @@ def collect_modules(options):
     if xtra(options.sqlite_ext):
         from playhouse.tests import test_sqlite_ext
         modules.append(test_sqlite_ext)
+    if xtra(options.sqlite_udf):
+        from playhouse.tests import test_sqlite_udf
+        modules.append(test_sqlite_udf)
     if xtra(options.test_utils):
         from playhouse.tests import test_test_utils
         modules.append(test_test_utils)
